-- TP 13B
-- fichier p_poly_v1_g-v2.adb
-- corrig du TP

with Ada.Exceptions;                use Ada.Exceptions;
with Ada.Strings.Unbounded.Text_Io; use Ada.Strings.Unbounded.Text_Io;
                                    use Ada.Strings.Unbounded;
with Ada.Strings.Maps;              use Ada.Strings.Maps;
                                    use Ada.Strings; 

package body P_Poly_V1_G.V2 is

   -- dbut des sous-programmes (amliors et surchargs version 1)
   -- un seul doit etre redfini compltement c'est "*"!
   -- beaucoup d'autres sont sur le modle de "+" ci dessous
   --
   function "+" (
         Poly_A,                    
         Poly_B : in     T_Polynome ) 
     return T_Polynome is 
   begin
      return P_Poly_V1_G."+"(Poly_A, Poly_B);
      -- contrle du dpassement de capacit du type des coefficients
   exception
      when Constraint_Error =>
         Raise_Exception (Exc_Debordement_Coef'Identity,
            "dpassement capacit des coefficients dans +");
   end "+";

   function "-" (
         Poly_A,                    
         Poly_B : in     T_Polynome ) 
     return T_Polynome is 
   begin
      return P_Poly_V1_G."-"(Poly_A, Poly_B);
      -- contrle du dpassement de capacit du type des coefficients
   exception
      when Constraint_Error =>
         Raise_Exception (Exc_Debordement_Coef'Identity,
            "dpassement capacit des coefficients dans -");
   end "-";

   function "-" (
         Poly_A : in     T_Polynome ) 
     return T_Polynome is 
   begin
      return P_Poly_V1_G."-"(Poly_A);
   exception
      when Constraint_Error =>
         Raise_Exception (Exc_Debordement_Coef'Identity,
            "dpassement capacit des coefficients dans -");
   end "-";

   function "*" (
         Poly_A,                    
         Poly_B : in     T_Polynome ) 
     return T_Polynome is 
   begin
      if Poly_A.Degre + Poly_B.Degre > Max
            then
         Raise_Exception (Exc_Depassement_Degre'Identity,
            "dpassement degr Max");
      end if;
      return   P_Poly_V1_G."*" (Poly_A,Poly_B);
   exception
      when Constraint_Error =>
         Raise_Exception (Exc_Debordement_Coef'Identity,
            "dpassement capacit des coefficients dans *");
   end "*";

   function "/" (
         Poly_A,                    
         Poly_B : in     T_Polynome ) 
     return T_Polynome is 
      Poly_Nul : T_Polynome;  
   begin
      if Poly_B = Poly_Nul
            then
         Raise_Exception (Exc_Division_Zero'Identity,
            "On ne divise pas par un polynome nul");
      else
         return P_Poly_V1_G."/" (Poly_A,Poly_B);
      end if;
   end "/";

   function "rem" (
         Poly_A,                    
         Poly_B : in     T_Polynome ) 
     return T_Polynome is 
      Poly_Nul : T_Polynome;  
   begin
      if Poly_B = Poly_Nul
            then
         Raise_Exception (Exc_Division_Zero'Identity,
            "On ne divise pas par un polynome nul");
      else
         return P_Poly_V1_G."rem" (Poly_A,Poly_B);
      end if;
   end "rem";


   function Derivee (
         Poly_A : in     T_Polynome ) 
     return T_Polynome is 
   begin
      return P_Poly_V1_G.Derivee (Poly_A);
      -- contrle du dpassement de capacit du type des coefficients
   exception
      when Constraint_Error =>
         Raise_Exception (Exc_Debordement_Coef'Identity,
            "dpassement capacit des coefficients dans Derivee");
   end Derivee;

   procedure Set (
         Poly :    out T_Polynome;  
         Vect : in     T_Liste_Coef ) is 
   begin
      P_Poly_V1_G.Set (Poly,Vect);
   exception
      when Constraint_Error =>
         Raise_Exception (Exc_Depassement_Degre'Identity,
            "dpassement degr Max dans Set");
   end Set;

   -- accesseurs ou slecteur :
   function Get_Degre (
         Poly : in     T_Polynome ) 
     return Natural is 
      Poly_Local : T_Polynome := Poly_Reduit (Poly);  
   begin
      return Poly_Local.Degre;
   end Get_Degre;

   function Get_Coef (
         Poly  : in     T_Polynome; 
         Degre : in     Natural     ) 
     return T_Rationnel is 
   begin
      if Degre > Max
            then
         Raise_Exception (Exc_Depassement_Degre'Identity,
            "degr suprieur  Max dans Get_Coef");
      end if;
      if Degre > Poly_Reduit (Poly).Degre
            then
         Raise_Exception (Exc_Degre_Inexistant'Identity,
            "degr non valide dans Get_Coef");
      else
         return Poly.Coef (Degre);
      end if;

   end Get_Coef;

   -- modifieur :
   procedure Set_Coef (
         Poly   : in out T_Polynome; 
         Degre  : in     Natural;    
         Valeur : in     T_Rationnel ) is 
      Poly_Local : T_Polynome := Poly_Reduit (Poly);  
   begin
      if Degre > Max
            then
         Raise_Exception (Exc_Depassement_Degre'Identity,
            "degr suprieur  Max dans Set_Coef");
      end if;
      if Degre > Poly_Local.Degre
            then
         Raise_Exception (Exc_Degre_Inexistant'Identity,
            "degr non valide dans Set_Coef");
      end if;
      Poly_Local.Coef (Degre) := Valeur;
      Poly := Poly_Local;
   end Set_Coef;

   procedure Get (
         Fich : in     File_Type; 
         Poly :    out T_Polynome ) is 

      Vect  : T_Liste_Coef (1 .. Max + 1);  
      L     : Natural                 := 0;  
      I     : Natural;  
      Ligne : Unbounded_String;  
      Fin   : Natural;  
   begin
      Ligne := Get_Line (Fich);
      loop
         I := Index (Ligne, To_Set("()"));
         exit when I = 0;
         Delete (Ligne,I,I);
      end loop;
      -- lecture d'une ligne de coef rationnels
      loop
         Trim (Ligne, Both );
         exit when Ligne = Null_Unbounded_String;
         L:= L + 1;
         Get (To_String(Ligne), Vect(L), Fin);
         Delete (Ligne,1,Fin);
      end loop;
      Set (Poly,Vect(1..L));
   exception
      when Constraint_Error =>
         Raise_Exception (Exc_Lecture'Identity,
            "problme constraint ");
      when Data_Error =>
         Raise_Exception (Exc_Lecture'Identity,
            "problme data ");
      when Exc_Depassement_Degre =>
         Raise_Exception (Exc_Lecture'Identity,
            "dpassement degr Max  ");
   end Get;

   -- autre
   function Valeur (
         Poly : in     T_Polynome; 
         X    : in     Float       ) 
     return Float is 
      Result : Float := 0.0;  
   begin
      for I in reverse Poly.Coef'range loop
         Result := Result * X+ Float(Numer(Poly.Coef(I)))
            / Float(Denom(Poly.Coef(I)));
      end loop;
      return Result;
      -- bonne rvision de HORNER et de la conversion
   end Valeur;

end P_Poly_V1_G.V2;
